%%%%%%%%%%%%%%%%%%  Designing sparse sensing matrix in
%%% "Optimized Sparse Projections for Compressive Sensing,"
%%%  https://arxiv.org/abs/1709.06895.  
%%%  by T. Hong, X. Li, Z. Zhu, and Qiuwei Li

function [ Phi,objec,relative_change ] = SparseSensing( A,Psi,k,lambda,M,Init);
%==========================================================================
%update the whole sparse sensing matrix, not one line by one line 
%--------------------------------------------------------------------------
%     Input:
%             - A, the DCT or Wavelet basis
%             - Psi, pre-trained dictionary 
%             - Init, initialization method for sensing matrix.
%             - k, sparsity in each row of sensing matrix 
%             - lambda, regularization param.
%             - M, compression size
%--------------------------------------------------------------------------
%     Output:
%             - Phi, sparse sensing mtx
%
%==========================================================================

%set param.
[N,L] = size(Psi);
if Init == 0
    Phi_iter = randn(M,N);
elseif Init == 1
    Phi_iter = ZRE(Psi,M);
else
    error('please specify an initialization method')
end
Gt = eye(L);
maxIter = 1000;
stepsize = 0.06;   
%pre-compute
D = A*Psi;
Id = eye(N);
% objec = []; relative_change = []; varstep = [];
objecfun = @(Phi) norm(Gt - D'*Phi'*Phi*D,'fro')^2 + lambda*norm(Phi*A,'fro')^2; %objective handle
%------------matlab pool------------
count = 1; stop = 0;
while ~stop 
    Phi_pre = Phi_iter;
    
    %gradient compute
    Phi_iter = sparse(Phi_iter);
    tmpPhiD = Phi_iter*D;
    if norm(A-Id,'fro')==0
        grad = -4*(tmpPhiD*Gt')*D' + 4*(tmpPhiD*tmpPhiD') *tmpPhiD *D'  ...
            + 2*lambda*Phi_iter;
    else
        grad = -4*(tmpPhiD*Gt')*D' + 4*(tmpPhiD*tmpPhiD') *tmpPhiD *D' ...
            + 2*lambda*(Phi_iter*A)*A';
    end
    
    %update
    if norm(grad,'fro') > 1
        %gradient descend
        Phi_iter = Phi_iter - (stepsize/norm(grad,'fro'))*grad;
    else
        %gradient descend
        Phi_iter = Phi_iter - stepsize*grad;
    end

    %thresholding
    for I = 1:size(Phi_iter,1)
        phi_hat = sort(abs(Phi_iter(I,:)),'descend');
        Phi_iter(I,:) = Phi_iter(I,:).* bsxfun(@ge,abs(Phi_iter(I,:)),phi_hat(k));
    end
    
    %compute objective
    objec(count) = objecfun(Phi_iter); 
    relative_change(count) = norm(Phi_iter - Phi_pre, 'fro') ;
    
    %determine stop or not
    if count >= maxIter || relative_change(end)< 1e-15
        stop = 1;
    end
    
count = count + 1;
end
Phi = Phi_iter*A;
end


